package io.tswf.minecraft.blockventure.core.inventory.screenhandler

import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.transforms.nullsafe.NullSafe
import it.unimi.dsi.fastutil.ints.IntOpenHashSet
import it.unimi.dsi.fastutil.ints.IntSet
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.item.ItemStack
import net.minecraft.screen.ScreenHandler

@CompileStatic
class MultipleSlotsQuickMoveHelper {

    Map<IntSet, StackInSlotQuickMoveHelper> slotsToQuickMoveHelpers = new LinkedHashMap<>()
    ScreenHandler screenHandler

    MultipleSlotsQuickMoveHelper(@NullSafe ScreenHandler screenHandler) {
        this.screenHandler = screenHandler
    }

    ItemStack quickMoveStack(PlayerEntity player, int slot) {
        def result = null
        for (def moveEntry : slotsToQuickMoveHelpers) {
            if (moveEntry.key.contains(slot)) {
                result = moveEntry.value.quickMoveStack(player, slot)
                if (result == null || result == ItemStack.EMPTY) {
                    break
                }
            }
        }

        return result ?: ItemStack.EMPTY
    }

    void addQuickMove(Iterable<Integer> slotsFrom, int slotTo) {
        addQuickMove(slotsFrom, Collections.singletonList(slotTo))
    }

    void addQuickMove(int slotFrom, Iterable<Integer> slotsTo) {
        addQuickMove(Collections.singletonList(slotFrom), slotsTo)
    }

    void addQuickMove(Iterable<Integer> slotsFrom, Iterable<Integer> slotsTo) {
        def slotsFromAsSet = new IntOpenHashSet()
        slotsFromAsSet.addAll(slotsFrom)

        slotsToQuickMoveHelpers[slotsFromAsSet] = new StackInSlotQuickMoveHelper(screenHandler, slotsTo)
    }
}
